/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "temphum5.h"

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void temphum5_cfg_setup ( temphum5_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->scl = HAL_PIN_NC;
    cfg->sda = HAL_PIN_NC;
    
    // Additional gpio pins

    cfg->all = HAL_PIN_NC;
    cfg->alh = HAL_PIN_NC;

    cfg->i2c_speed = I2C_STANDARD_SPEED; 
    cfg->i2c_address = TEMPHUM5_DEVICE_SLAVE_ADDRESS;

    cfg->dev_temperature = 0;
    cfg->dev_humidity = 0;
}

TEMPHUM5_RETVAL temphum5_init ( temphum5_t *ctx, temphum5_cfg_t *cfg )
{
     i2c_master_config_t i2c_cfg;

    i2c_master_configure_default( &i2c_cfg );
    i2c_cfg.speed    = cfg->i2c_speed;
    i2c_cfg.pins.scl = cfg->scl;
    i2c_cfg.pins.sda = cfg->sda;

    ctx->slave_address = cfg->i2c_address;

    ctx->temperature = cfg->dev_temperature;
    ctx->humidity = cfg->dev_humidity;

    if ( i2c_master_open( &ctx->i2c, &i2c_cfg ) != I2C_SUCCESS )
    {
        return TEMPHUM5_INIT_ERROR;
    }

    // Input pins

    digital_in_init( &ctx->all, cfg->all );
    digital_in_init( &ctx->alh, cfg->alh );
    
    return TEMPHUM5_OK;
}

void temphum5_generic_write ( temphum5_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 256 ];
    uint8_t cnt;
    
    tx_buf[ 0 ] = reg;
    
    for ( cnt = 1; cnt <= len; cnt++ )
    {
        tx_buf[ cnt ] = data_buf[ cnt - 1 ]; 
    }
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = len + 1;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );    
}

void temphum5_generic_read ( temphum5_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 1 ];

    tx_buf [ 0 ] = reg;
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = 1;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );
    
    i2c_transfer.buffer   = data_buf;
    i2c_transfer.count    = len;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    Delay_100ms();
    
    i2c_master_start( &ctx->i2c );
    i2c_master_read( &ctx->i2c, &i2c_transfer );
}

uint8_t temphum5_start_measurement ( temphum5_t *ctx )
{
    uint8_t read_reg[ 4 ];
    uint8_t status;
    uint8_t command = 0x00;
    
    temphum5_generic_read( ctx, command, read_reg, 4 );

    ctx->humidity = read_reg[ 0 ];
    ctx->humidity <<= 8;
    ctx->humidity |= read_reg[ 1 ];
    ctx->humidity = ctx->humidity & 0x3FFF;

    ctx->temperature = read_reg[ 2 ];
    ctx->temperature <<= 8;
    ctx->temperature |= read_reg[ 3 ];
    ctx->temperature = ( ctx->temperature >> 2 ) & 0x3FFF;
    
    status = ( read_reg[ 0 ] & 0x3F ) >> 6;
    
    if ( status == TEMPHUM5_STATUS_STALE_DATA )
    {
        return TEMPHUM5_STATUS_STALE_DATA;
    }
    else if ( status == TEMPHUM5_STATUS_COMMAND_MODE )
    {
        return TEMPHUM5_STATUS_COMMAND_MODE;
    }
    return TEMPHUM5_STATUS_NORMAL_OPERATION;
}

float temphum5_get_temperature ( temphum5_t *ctx, uint8_t temp_in )
{
    float temp;
    
    temp = ( float ) ( ctx->temperature / 16382.0 ) * 165.0 - 40;
    
    if ( temp_in == TEMPHUM5_TEMP_DATA_IN_KELVIN )
    {
        return temp + 273.15;
    }
    else if ( temp_in == TEMPHUM5_TEMP_DATA_IN_FAHRENHEIT )
    {
        return ( temp * 9.0/5.0 ) + 32;
    }
    return temp;
}
float temphum5_get_humidity ( temphum5_t *ctx )
{
    float hum;
    
    hum = ( float ) ( ctx->humidity / 16382.0 ) * 100.0;
    return hum;
}

uint8_t temphum5_get_alarm ( temphum5_t *ctx, uint8_t alarm )
{
    if ( alarm == TEMPHUM5_ALARM_OUTPUT_HIGH )
    {
        return digital_in_read( &ctx->alh );
    }
    else
    {
        return digital_in_read( &ctx->all );
    }
}

// ------------------------------------------------------------------------- END

